@@ -26,10 +26,13 @@
 typedef struct _GritsHttp {
 	SoupSession *soup;
 	gchar *prefix;
+	gboolean aborted;
 } GritsHttp;
 
 GritsHttp *grits_http_new(const gchar *prefix);
 
+void grits_http_abort(GritsHttp *http);
+
 void grits_http_free(GritsHttp *http);
 
 gchar *grits_http_fetch(GritsHttp *http, const gchar *uri, const gchar *local,
