@@ -22,6 +22,14 @@ extern void grits_cclosure_marshal_VOID__STRING_UINT_POINTER (GClosure     *clos
                                                               gpointer      invocation_hint,
                                                               gpointer      marshal_data);
 
+/* BOOLEAN:POINTER (grits-marshal.list:3) */
+extern void grits_cclosure_marshal_BOOLEAN__POINTER (GClosure     *closure,
+                                                     GValue       *return_value,
+                                                     guint         n_param_values,
+                                                     const GValue *param_values,
+                                                     gpointer      invocation_hint,
+                                                     gpointer      marshal_data);
+
 G_END_DECLS
 
 #endif /* __grits_cclosure_marshal_MARSHAL_H__ */
