@@ -159,6 +159,9 @@ struct _GritsPoint {
 void grits_point_set_lle(GritsPoint *point,
 		gdouble lat, gdouble lon, gdouble elev);
 
+/* GritsPoints */
+typedef gdouble (*GritsPoints)[3];
+
 /* GritsBounds */
 typedef struct _GritsBounds GritsBounds;
 struct _GritsBounds {
@@ -177,7 +180,7 @@ void grits_bounds_set_bounds(GritsBounds *bounds,
  *
  * Used by GritsOpenGL to set up the drawing window
  */
-#define FOV_DIST   2000.0
+#define FOV_DIST   (1039.23)
 
 /**
  * MPPX:
@@ -187,7 +190,7 @@ void grits_bounds_set_bounds(GritsBounds *bounds,
  *
  * Returns: the resolution in meters per pixel
  */
-#define MPPX(dist) (4*dist/FOV_DIST)
+#define MPPX(dist) (dist/FOV_DIST)
 
 void lle2xyz(gdouble lat, gdouble lon, gdouble elev,
 		gdouble *x, gdouble *y, gdouble *z);
@@ -212,4 +215,10 @@ void normd(gdouble *a);
 
 gdouble lon_avg(gdouble a, gdouble b);
 
+GritsPoints *parse_points(const gchar *string,
+		const gchar *group_sep, const gchar *point_sep, const gchar *coord_sep,
+		GritsBounds *bounds, GritsPoint *center);
+
+void free_points(GritsPoints *points);
+
 #endif
