@@ -27,6 +27,7 @@
 /* Grits data */
 #include <data/grits-data.h>
 #include <data/grits-http.h>
+#include <data/grits-tms.h>
 #include <data/grits-wms.h>
 
 /* Grits objects */
@@ -36,8 +37,12 @@
 #include <objects/grits-callback.h>
 #include <objects/grits-volume.h>
 #include <objects/grits-poly.h>
+#include <objects/grits-line.h>
 
 /* Plugins */
 #include <grits-plugin.h>
 
+/* Misc */
+#include <gtkgl.h>
+
 #endif
