@@ -23,12 +23,18 @@
 #include "data/grits-tms.h"
 #include "objects/grits-tile.h"
 
+#include "compat.h"
+
 struct CacheState {
 	GtkWidget *image;
 	GtkWidget *status;
 	GtkWidget *progress;
 };
 
+struct LoadData {
+	GtkImage  *image;
+	GdkPixbuf *pixbuf;
+};
 
 void chunk_callback(gsize cur, gsize total, gpointer _state)
 {
@@ -47,6 +53,22 @@ void chunk_callback(gsize cur, gsize total, gpointer _state)
 		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(state->progress), (gdouble)cur/total);
 }
 
+gboolean load_callback(gpointer _data)
+{
+	struct LoadData *data = _data;
+	gtk_image_set_from_pixbuf(GTK_IMAGE(data->image), data->pixbuf);
+	g_free(data);
+	return FALSE;
+}
+
+void load_image(GtkImage *image, GdkPixbuf *pixbuf)
+{
+	struct LoadData *data = g_new0(struct LoadData, 1);
+	data->image  = image;
+	data->pixbuf = pixbuf;
+	g_idle_add(load_callback, data);
+}
+
 gpointer do_bmng_cache(gpointer _image)
 {
 	GtkImage *image = _image;
@@ -63,9 +85,7 @@ gpointer do_bmng_cache(gpointer _image)
 
 	g_message("Loading bmng image: [%s]", path);
 	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file(path, NULL);
-	gdk_threads_enter();
-	gtk_image_set_from_pixbuf(GTK_IMAGE(image), pixbuf);
-	gdk_threads_leave();
+	load_image(GTK_IMAGE(image), pixbuf);
 
 	g_message("Cleaning bmng up");
 	grits_wms_free(bmng_wms);
@@ -89,9 +109,7 @@ gpointer do_osm_cache(gpointer _image)
 
 	g_message("Loading osm image: [%s]", path);
 	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file(path, NULL);
-	gdk_threads_enter();
-	gtk_image_set_from_pixbuf(GTK_IMAGE(image), pixbuf);
-	gdk_threads_leave();
+	load_image(GTK_IMAGE(image), pixbuf);
 
 	g_message("Cleaning osm up");
 	grits_wms_free(osm_wms);
@@ -114,9 +132,7 @@ gpointer do_osm2_cache(gpointer _image)
 
 	g_message("Loading osm2 image: [%s]", path);
 	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file(path, NULL);
-	gdk_threads_enter();
-	gtk_image_set_from_pixbuf(GTK_IMAGE(image), pixbuf);
-	gdk_threads_leave();
+	load_image(GTK_IMAGE(image), pixbuf);
 
 	g_message("Cleaning osm2 up");
 	grits_tms_free(osm2_tms);
@@ -127,20 +143,18 @@ gpointer do_osm2_cache(gpointer _image)
 
 gboolean key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
 {
-	if (event->keyval == GDK_q)
+	if (event->keyval == GDK_KEY_q)
 		gtk_main_quit();
 	return TRUE;
 }
 
 int main(int argc, char **argv)
 {
-	g_thread_init(NULL);
-	gdk_threads_init();
 	gtk_init(&argc, &argv);
 
 	GtkWidget *win        = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	GtkWidget *vbox1      = gtk_vbox_new(FALSE, 0);
-	GtkWidget *vbox2      = gtk_vbox_new(FALSE, 0);
+	GtkWidget *vbox1      = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+	GtkWidget *vbox2      = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	GtkWidget *status     = gtk_statusbar_new();
 	GtkWidget *scroll     = gtk_scrolled_window_new(NULL, NULL);
 	GtkWidget *bmng_image = gtk_image_new();
@@ -160,9 +174,9 @@ int main(int argc, char **argv)
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
 			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
-	g_thread_create(do_bmng_cache, bmng_image, FALSE, NULL);
-	g_thread_create(do_osm_cache,  osm_image,  FALSE, NULL);
-	g_thread_create(do_osm2_cache, osm2_image, FALSE, NULL);
+	g_thread_new("bmng-thread", do_bmng_cache, bmng_image);
+	g_thread_new("osm-thread",  do_osm_cache,  osm_image);
+	g_thread_new("osm2-thread", do_osm2_cache, osm2_image);
 
 	gtk_widget_show_all(win);
 	gtk_main();
